const db = require('./src/db');
const protoBuf = require('pbf');
const debug = require('debug')('index');
require('dotenv').config();

const gtfsRt = require('../../proto2js/js/gtfs-rt.js');

const gtfsrFetch = require('./src/fetch');
const tUO = require('./src/trip-update-obj');

const timeoutScnds = 60;

async function timeoutFct()
{
    console.log('index:setInterval start...');

    //initialize array of trip update objects
    const rryOfTripUpdateObjcs = new Array();

    //fetch GTFS Realtime feed and store trip updates as array of objects
    let buffer = null
    buffer = await gtfsrFetch.fetch();

    if (buffer !== null && buffer !== undefined) {
	//TODO clean up debug('index:intervalFunc(): buffer available');
        const FeedMessage = gtfsRt.FeedMessage;
	const prtBf = new protoBuf(buffer);
	const feed = FeedMessage.read(prtBf);
	let countTrip = 0;
	let countTripId = 0;
	let countRouteId = 0;
	let countScheduleRelationship = 0;
	let countVehicle = 0;
	let countTimestamp = 0;
	let countDelay = 0;
	feed.entity.forEach(entity => {
            const entityTripUpdate = entity.trip_update;
            if (entityTripUpdate !== null && entityTripUpdate !== undefined) {
		const tuo = new tUO.TripUpdate();
		const trip = entityTripUpdate.trip;
		if (trip !== null && trip !== undefined) {
		    countTrip++
		    const t = new tUO.Trip();
		    const tripId = trip.trip_id;
		    if (tripId !== null && tripId !== undefined) {
			countTripId++;
			t.tripId = tripId;
		    }
		    const routeId = trip.route_id;
		    if (routeId !== null && routeId !== undefined) {
			countRouteId++;
			t.routeId = routeId;
		    }
		    const scheduleRelationship = trip.schedule_relationship;
		    if (scheduleRelationship !== null && scheduleRelationship !== undefined) {
			countScheduleRelationship++;
			t.scheduleRelationship = scheduleRelationship;
		    }
		    //create a copy of an object, you can use the spread operator
		    tuo.trip = {...t};
		}
		const vehicle = entityTripUpdate.vehicle;
		if (vehicle !== null && vehicle !== undefined) {
		    countVehicle++;
		}
		const timestamp = entityTripUpdate.timestamp;
		if (timestamp !== null && timestamp !== undefined) {
		    countTimestamp++;
		    tuo.timestamp = timestamp;
		}
		const delay = entityTripUpdate.delay;
		if (delay !== null && delay !== undefined) {
		    countDelay++;
		    tuo.delay = delay;
		}
		rryOfTripUpdateObjcs.push({...tuo});
            }
	});
	debug('index:intervalFunc(): rryOfTripUpdateObjcs.length: ' + rryOfTripUpdateObjcs.length);
	//TODO clean up
	/*
	debug('index:intervalFunc(): countTrip: ' + countTrip);
	debug('index:intervalFunc(): countTripId: ' + countTripId);
	debug('index:intervalFunc(): countRouteId: ' + countRouteId);
	debug('index:intervalFunc(): countScheduleRelationship: ' + countScheduleRelationship);
	debug('index:intervalFunc(): countVehicle: ' + countVehicle);
	debug('index:intervalFunc(): countTimeStamp: ' + countTimestamp);
	debug('index:intervalFunc(): countDelay: ' + countDelay);
	*/
    } else {
	console.error('ERROR: index:intervalFunc(): buffer NOT available');
    }

    //select trip updates of today from db as array of objects
    const schema = process.env.DB_SCHEMA || 'schema';

    //TODO some GTFSR feeds do not provide timestamp const query = 'SELECT * FROM ' + schema + '.trip_updates WHERE timestamp_gtfsr >= current_date';
    //NOTE: https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-CURRENT
    const query = 'SELECT * FROM ' + schema + '.trip_updates WHERE timestamp_pgsql >= current_date';

    //TODO clean up debug('query: '+query);
    const rsp = await db.query(query);
    debug('rsp.length: '+rsp.length);
    if (rsp !== null && rsp !== undefined && rsp.length < 0) {
	//TODO clean up
	debug('rsp[0]: ' + JSON.stringify(rsp[0]));
    }

    //TODO only continue if rsp is neither null or undefined

    //transform array of objects into map [trip_trip_id, trip update object]
    //TODO handle rsp NOT available
    const mapDbTripUpdates = new Map();
    rsp.forEach(element => {
	mapDbTripUpdates.set(element.trip_trip_id, element);
    });
    
    debug('mapDbTripUpdates.size: ' + mapDbTripUpdates.size);

    //compare map from db with array of trip update objcs from GTFS Realtime feed
    //TODO clean up
    let countRryElem = 0;
    let countInsert = 0;
    //TODO https://stackoverflow.com/a/37576787/15078958
    //TODO according to this link forEach() has challenges for async/await
    for (const element of rryOfTripUpdateObjcs) {
	const value = mapDbTripUpdates.get(element.trip.tripId);
	let query = '';
	let rspPsql = null;

	if (value === undefined) {
	    query = `INSERT INTO `
		+ schema
		+ `.trip_updates (trip_trip_id, trip_route_id, trip_schedule_relationship, timestamp_gtfsr, delay) VALUES ('`
		+ element.trip.tripId
		+ `', '`
		+ element.trip.routeId
		+ `', '`
		+ element.trip.scheduleRelationship
		+ `', to_timestamp(`
		+ element.timestamp
		+ `), '`
		+ element.delay
		+ `');`;
	    rspPsql = await db.query(query);
	    countInsert++;

	    if (countRryElem === 0) {
	    debug('value: ' + JSON.stringify(value));
	    debug('query: ' + query);
	    debug('respDb: ' + JSON.stringify(rspPsql));
	    }
	    countRryElem++;
	}
    }

    debug('countInsert: ' + countInsert);

    //other
    console.log('index:setInterval done.');
    
    setTimeout(timeoutFct, timeoutScnds * 1000);
}

async function run() {
    debug('index:run(): started...');

    timeoutFct();
};

run().catch(err => {
    console.error('ERROR: ');
    console.log(err)
});
